import React, { useEffect, useRef, useState } from 'react';
import {
  useAppDispatch,
  SaveStateI,
  wsUpdateState,
  wsTestDatabaseConnection,
  wsCreateState,
  useAuthorizationCache,
} from '@graphpolaris/shared/lib/data-access';
import { ErrorOutline } from '@mui/icons-material';
import { Dialog } from '@graphpolaris/shared/lib/components/Dialog';
import { Button } from '@graphpolaris/shared/lib/components/buttons';
import { useImmer } from 'use-immer';
import { addSaveState, testedSaveState } from '@graphpolaris/shared/lib/data-access/store/sessionSlice';
import { DatabaseForm, INITIAL_SAVE_STATE } from './databaseForm';
import { SampleDatabaseSelector } from './mockSaveStates';

type Connection = {
  updating: boolean;
  status: null | string;
  verified: boolean | null;
};

export const SettingsForm = (props: { onClose(): void; open: 'add' | 'update'; saveState: SaveStateI | null; disableCancel?: boolean }) => {
  const dispatch = useAppDispatch();
  const ref = useRef<HTMLDialogElement>(null);
  const auth = useAuthorizationCache();
  const [formData, setFormData] = useImmer(
    props.saveState && props.open === 'update' ? props.saveState : { ...INITIAL_SAVE_STATE, user_id: auth.userID || '' },
  );
  const [hasError, setHasError] = useState(false);
  const [sampleDataPanel, setSampleDataPanel] = useState<boolean | null>(false);
  const [connection, setConnection] = useState<Connection>({
    updating: false,
    status: null,
    verified: null,
  });
  const formTitle = props.open === 'add' ? 'Add' : 'Update';

  useEffect(() => {
    if (props.saveState && props.open === 'update') {
      setFormData(props.saveState);
      setSampleDataPanel(null);
    } else {
      setSampleDataPanel(false);
    }
  }, [props.saveState]);

  async function handleSubmit(saveStateData?: SaveStateI) {
    if (!saveStateData) saveStateData = formData;
    setConnection(() => ({
      updating: true,
      status: 'Testing database connection',
      verified: null,
    }));

    wsTestDatabaseConnection(saveStateData.db, (data) => {
      if (!saveStateData) {
        console.error('formData is null');
        return;
      }
      if (saveStateData.user_id !== auth.userID && auth.userID) {
        console.error('user_id is not equal to auth.userID');
        saveStateData.user_id = auth.userID;
      }
      if (data && data.status === 'success') {
        setConnection((prevState) => ({
          updating: false,
          status: 'Database connection verified',
          verified: true,
        }));
        if (props.open === 'add') {
          wsCreateState(saveStateData, (_data) => {
            dispatch(addSaveState(_data));
            dispatch(testedSaveState(_data.id));
            closeDialog();
          });
        } else {
          dispatch(testedSaveState(data.saveStateID));
          wsUpdateState(saveStateData, (_data) => {
            dispatch(addSaveState(_data));
            closeDialog();
          });
        }
      } else {
        setConnection((prevState) => ({
          updating: false,
          status: 'Database connection test failed',
          verified: false,
        }));
      }
    });
  }

  function handlePortChanged(port: string): void {
    if (!isNaN(Number(port)))
      setFormData((draft) => {
        draft.db.port = Number(port);
        return draft;
      });
  }

  function closeDialog(): void {
    setConnection({
      updating: false,
      status: null,
      verified: null,
    });
    setFormData({ ...INITIAL_SAVE_STATE, user_id: auth.userID || '' });
    ref.current?.close();
    props.onClose();
  }

  return (
    <Dialog open={!!props.open} onClose={props.onClose} className="lg:min-w-[50rem]">
      <div className="flex justify-between align-center">
        <h1 className="text-xl font-bold">{formTitle} Database Connection</h1>
        <div>
          {sampleDataPanel === true ? (
            <Button variant="outline" label="Go back" onClick={() => setSampleDataPanel(false)} />
          ) : sampleDataPanel === false ? (
            <>
              <h1 className="font-light text-xs">No data?</h1>
              <p className="font-light text-sm cursor-pointer underline" onClick={() => setSampleDataPanel(true)}>
                Try sample data
              </p>
            </>
          ) : (
            ''
          )}
        </div>
      </div>

      <>
        {sampleDataPanel === true ? (
          <SampleDatabaseSelector
            onClick={(data) => {
              setHasError(false);
              handleSubmit({ ...data, user_id: auth.userID || '' });
            }}
          />
        ) : (
          <DatabaseForm
            data={formData}
            onChange={(data: SaveStateI, error: boolean) => {
              setFormData({ ...data, id: formData.id });
              setHasError(error);
            }}
          />
        )}

        {!(connection.status === null) && (
          <div className={`flex flex-col justify-center items-center`}>
            <div className="flex justify-center items-center">
              {connection.verified === false && <ErrorOutline className="text-secondary-400" />}
              <p className="font-light text-sm	text-secondary-400	">{connection.status}</p>
            </div>
            {connection.verified === null && <progress className="progress w-56"></progress>}
          </div>
        )}

        <div
          className={`grid md:grid-cols-2 gap-3 card-actions w-full justify-stretch items-center ${sampleDataPanel === true && 'hidden'}`}
        >
          <Button
            type="primary"
            label={connection.updating ? formTitle.slice(0, -1) + 'ing...' : formTitle}
            onClick={(event) => {
              event.preventDefault();
              handleSubmit();
            }}
            disabled={connection.updating || hasError}
          />
          <Button
            variant="outline"
            label="Cancel"
            disabled={props.disableCancel}
            onClick={(event) => {
              event.preventDefault();
              closeDialog();
            }}
          />
        </div>
      </>
    </Dialog>
  );
};
